'use strict';

(function (){

  function preload($q) {
    return {
      restrict: 'EA',
      template: [
        '<div class="preloading">',
          '<div ng-if="!ready"><span>{{message}}</span></div>',
          '<div ng-if="ready" ng-transclude></div>',
        '</div>'
      ].join(''),
      replace: true,
      transclude: true,
      scope: {
        load: '=',
        loadingMessage: '@',
        errorMessage: '@'
      },
      link: function (scope, element, attrs) {

        var isStringArray = attrs.load.substr(0,1) === '[' && attrs.load.substr(-1,1) === ']',
            isEmpty = scope.load === undefined || (isStringArray && attrs.load.length <= 2),
            promiseArray = [];

        var errorHandler = function(msg) {
          scope.message = msg;
          element.removeClass('preloading').addClass('error');
        };

        scope.ready = false;
        scope.message = scope.loadingMessage || 'Loading...';

        if (isEmpty) {
          errorHandler('Error: No data specified.');
          return;
        }

        if (!isEmpty && isStringArray) {
          for (var i = 0; i < scope.load.length; i++) {
            if (scope.load[i] === undefined) {
              errorHandler('Error: Data is not defined in current scope.');
              return;
            }
          }
          promiseArray = promiseArray.concat(scope.load);
        } else if (!isEmpty && !isStringArray) {
          promiseArray.push(scope.load);
        } else {
          return;
        }

        $q.all(promiseArray).then(
          function() {
            element.removeClass('preloading').addClass('preloaded');
            scope.ready = true;
          },
          function() {
            errorHandler(scope.errorMessage || 'Error: Data failed to load.');
          }
        );
      }
    };
  }

  angular
    .module('angular-preloader', [])
    .directive('preload', ['$q', preload]);

})();